package middleware.agents;

import java.io.Serializable;
import java.security.InvalidParameterException;

/**
 * Agent Address
 * Addresses for metaAgents
 * 
 * @author JLScott1999
 */
public final class AgentAddress implements Serializable
{
    
    private int[] address;
    
    /**
     * Creates an instance of AgentAddress
     * @param sections no. of sections in address
     * @param addressString address
     */
    public AgentAddress(int sections, String addressString)
    {
        address = new int[sections];
        setAddress(addressString);
    }
    
    /**
     * Creates a new instance of AgentAddress
     * @param sections no. of sections in address
     */
    public AgentAddress(int sections)
    {
        this(sections, null);
    }
    
    /**
     * Creates a new instance of AgentAddress
     * Default of 3 sections
     * @param addressString address string
     */
    public AgentAddress(String addressString)
    {
        this(3, addressString);
    }
    
    /**
     * Creates instance of AgentAddress with a 3 section address
     */
    public AgentAddress()
    {
        this(3);
    }
    
    /**
     * Sets valid address based off of given string
     * @param addressString addressString
     */
    public void setAddress(String addressString)
    {
        if (addressString != null)
        {
            int i = 0;
            int section = 0;
            while (i < addressString.length())
            {
                int dot = addressString.indexOf('.', i);
                if (dot != -1)
                {
                    address[section++] = Integer.parseInt(addressString.substring(i, dot));
                    i = dot + 1;
                }
                else
                {
                    address[section] = Integer.parseInt(addressString.substring(i, addressString.length()));
                    i = addressString.length();
                }
            }
        }
    }
    
    /**
     * Gets Address
     * @return address
     */
    public String getAddress()
    {
        return toString();
    }
    
    /**
     * Changes value of address  at given section
     * @param section specified section
     * @param value new value
     */
    public void setSection(int section, int value)
    {
        if (inSectionRange(section))
        {
            address[section - 1] = value;
        }
        else
        {
            throw new InvalidParameterException();
        }
    }
    
    private boolean inSectionRange(int section)
    {
        return (section > 0 && section <= address.length);
    }
    
    /**
     * 
     * @param section
     * @return 
     */
    public int getSection(int section)
    {
        if (inSectionRange(section))
        {
            return address[section - 1];
        }
        else
        {
            throw new InvalidParameterException();
        }
    }
    
    @Override
    public String toString()
    {
        String result = "";
        for (int i = 0; i < address.length; i++)
        {
            result += address[i] + ((address.length == i + 1 ? "" : "."));
        }
        return result;
    }
    
}