package middleware.agents;

import java.io.IOException;
import java.net.Inet4Address;
import java.net.UnknownHostException;
import java.util.HashMap;
import middleware.message.MetaAgentMessage;
import middleware.message.SystemErrorMessage;
import middleware.message.UserMessage;
import middleware.message.wrapper.MetaAgentWrapper;
import middleware.socket.Client;
import middleware.socket.Server;

/**
 * MetaAgent for Routers allowing for communication between different routers
 *
 */
public class Router extends MetaAgent {

    int port;
    Inet4Address ipAddress;
    private Server listener;
    private HashMap<String, Client> sender = new HashMap();

    /**
     * Creates a new instance of MetaAgent Router with given properties
     * @param queueSize size of blocking queue
     * @param address router address
     * @param ipAddressIn ip address
     * @param portIn port
     * @throws java.net.UnknownHostException
     */
    public Router(int queueSize, String address, int portIn) throws UnknownHostException {
        super(queueSize, address);
        ipAddress = (Inet4Address) Inet4Address.getByName(address);
        port = portIn;
        listener = new Server(portIn, this);
        try
        {
            listener.create();
            listener.enableListening();
        } catch (IOException ex)
        {
            ex.printStackTrace();
        }
    }

    /**
     * Adds a message to the queue
     * @param e message
     * @return success
     */
    @Override
    public boolean add(Object e)
    {
        if (e instanceof UserMessage)
        {
            return super.add(MetaAgentWrapper.wrap((UserMessage) e, 15));
        }
        else if (e instanceof MetaAgentMessage)
        {
            return super.add(e);
        }
        return false;
    }

    /**
     * Creates a connection using specified parameters
     * @param addressString
     * @param address
     * @param port
     */
    public void connect(String addressString, String address, int port)
    {
        Client client = new Client(address, port);
        System.out.println(address + " + " + port);
        sender.put(addressString, client);
        try
        {
            client.connect();
        } catch (IOException ex)
        {
            ex.printStackTrace();
        }

    }

    /**
     * Handles message objects passed to it
     * @param data message
     */
    @Override
    public void msgHandler(Object data)
    {
        if (data instanceof MetaAgentMessage)
        {
            MetaAgentMessage MAM = (MetaAgentMessage) data;
            try
            {
                sender.get(MAM.getMessage().getRecipient().getSection(1) + ".0.0").send((MetaAgentMessage) data);
                super.updateObservers(data);
            }
            catch (Exception e)
            {
                if (MAM.timeToLive > 0)
                {
                    MAM.decreaseTimeToLive();
                    MAM.increaseHops();
                    super.msgHandler(MAM);
                }
                else
                {
                    if (!MAM.getMessage().getSender().getAddress().equals("0.0.0"))
                    {
                        super.msgHandler(MetaAgentWrapper.wrap(new SystemErrorMessage(MAM.getMessage().getRecipient().getAddress() + " Does not exist?", MAM.getMessage().getSender().getAddress()), 15));
                    }
                }

            }
        }
    }

}
