package middleware.agents;

import java.util.ArrayList;
import middleware.message.Message;
import middleware.message.UserMessage;

/**
 * Agent for a user/node/endpoint
 * @author JLScott1999
 */
public class UserAgent
{
    private String address;
    private MetaAgent metaAgent;
    private ArrayList<MsgListener> listeners = new ArrayList();
    
    /**
     * Creates an instance of UserAgent with a specified address
     * @param address Specified address
     */
    public UserAgent(String address)
    {
        this.address = address;
    }
    
    /**
     * Returns the UserAgent's address
     * @return address
     */
    public String getAddress()
    {
        return address;
    }
    
    /**
     * Used to wrap and send a message to a recipient
     * @param sender the sender
     * @param recipient the message recipient
     * @param message the message content to be sent 
     */
    public void sendMsg(String sender, String recipient, String message)
    {
        metaAgent.add(new UserMessage(message, recipient, sender, false, "???", "0"));
    }
    
    /**
     * Set's the UserAgent's MetaAgent (e.g. the portal it belongs to)
     * @param metaAgent metaAgent to be assigned
     */
    public void setMetaAgent(MetaAgent metaAgent)
    {
        this.metaAgent = metaAgent;
    }
    
    /**
     * Used by msgHandler to receive message
     * @param data message received
     */
    public void recieveMsg(Message data)
    {
        for (MsgListener listener : listeners)
        {
            listener.UserAgentMsg(data.getSender().getAddress().toString(), data.getMessage().toString());
        }
    }
    
    /**
     * Adds a message listener to the UserAgent
     * @param listener Message Listener
     */
    public void addListener(MsgListener listener)
    {
        listeners.add(listener);
    }
    
}