
package middleware.components.main.nodelistener;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

/**
 * Used to display output from a NodeMonitor
 * 
 * @author Dan Lawrence
 */
public class NodeListener extends JTable {
    DefaultTableModel model;
    
    /**
     * Sets up GUI components for NodeListener
     */
    public NodeListener() {
        setUpComponent();
    }
    
    private void setUpComponent() {
        //TTL: Time to live
        String[] headings = {"Time sent/received","Sender","Receiver","Message","TTL","Hops"};
        
        model = new DefaultTableModel(headings,0) {
            @Override
            public boolean isCellEditable(int row, int column) {
                return false;
            }
        };
        
        setModel(model);
        
        setFocusable(false);
        setRowSelectionAllowed(false);
               
        getColumnModel().getColumn(0).setPreferredWidth(130);
        getColumnModel().getColumn(0).setMaxWidth(130);
        getColumnModel().getColumn(0).setMinWidth(130);
        getColumnModel().getColumn(0).setResizable(false);
        
        getColumnModel().getColumn(1).setPreferredWidth(100);
        getColumnModel().getColumn(1).setMaxWidth(100);
        getColumnModel().getColumn(1).setMinWidth(100);
        getColumnModel().getColumn(1).setResizable(false);
        
        getColumnModel().getColumn(2).setPreferredWidth(100);
        getColumnModel().getColumn(2).setMaxWidth(100);
        getColumnModel().getColumn(2).setMinWidth(100);
        getColumnModel().getColumn(2).setResizable(false);
        
        getColumnModel().getColumn(3).setMinWidth(450);
        getColumnModel().getColumn(3).setResizable(true);
        
        getColumnModel().getColumn(4).setPreferredWidth(50);
        getColumnModel().getColumn(4).setMaxWidth(50);
        getColumnModel().getColumn(4).setMinWidth(50);
        getColumnModel().getColumn(4).setResizable(false);
        
        getColumnModel().getColumn(5).setPreferredWidth(50);
        getColumnModel().getColumn(5).setMaxWidth(50);
        getColumnModel().getColumn(5).setMinWidth(50);
        getColumnModel().getColumn(5).setResizable(false);
        
        setRowHeight(20);
        
        setVisible(true);
        setOpaque(true);
    }
    
    /**
     * Adds a row to the NodeListener Table, using unwrapped message parameters
     * @param message message content
     * @param sender sender
     * @param recipient recipient
     * @param ttl Time To Live (TTL)
     * @param hops message hops
     */
    public void addRow(String message, String sender, String recipient, int ttl, int hops) {
        Object[] tableRow = new Object[6];
            
        tableRow[0] = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss").format(Calendar.getInstance().getTime());
        tableRow[1] = sender;
        tableRow[2] = recipient;
        tableRow[3] = message;
        tableRow[4] = ttl;
        tableRow[5] = hops;
        
        model.addRow(tableRow);
    }
    
    /**
     * Clears NodeListenerTable
     */
    public void clear() {
        model.setRowCount(0);
    }
}
