
package middleware.components.useragents;

import java.awt.BorderLayout;
import java.awt.Dimension;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import middleware.agents.MsgListener;
import middleware.agents.UserAgent;
import middleware.components.useragents.messagefeed.MessageFeed;
import middleware.components.useragents.messagesending.MessageSender;

/**
 * The GUI for UserAgents
 *
 * @author Dan Lawrence
 */
public class UserAgentUI extends JFrame implements MsgListener
{

    private final MessageSender messageSender;
    private final MessageFeed messageFeed;
    private final UserAgent userAgent;

    /**
     * Sets up GUI for a UserAgent
     * @param userAgent specified UserAgent
     */
    public UserAgentUI(UserAgent userAgent) {
        messageFeed = new MessageFeed();
        this.userAgent = userAgent;
        messageSender = new MessageSender(userAgent, messageFeed);
        userAgent.addListener(this);
        setUpComponent();
    }

    private void setUpComponent() {
        setTitle("User Agent - " + userAgent.getAddress().toUpperCase());

        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        setVisible(true);

        JPanel padding1 = new JPanel();
        padding1.setPreferredSize(new Dimension(10,0));

        JPanel padding2 = new JPanel();
        padding2.setPreferredSize(new Dimension(10,0));

        add(padding1, BorderLayout.WEST);
        add(padding2, BorderLayout.EAST);

        JScrollPane messageFeedScroller = new JScrollPane(messageFeed);
        messageFeedScroller.setPreferredSize(new Dimension(780,200));

        add(messageFeedScroller, BorderLayout.CENTER);
        add(messageSender, BorderLayout.SOUTH);

        pack();
        setMinimumSize(getSize());
    }

    @Override
    public void UserAgentMsg(String sender, String msg)
    {
        messageFeed.addRow(msg, sender.toUpperCase(), userAgent.getAddress().toUpperCase());
    }
}
