package middleware.components.useragents.messagefeed;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

/**
 * MessageFeed for UserAgents
 *
 * @author Dan Lawrence
 */
public class MessageFeed extends JTable {

    DefaultTableModel model;

    /**
     * Sets up GUI for MessageFeed
     */
    public MessageFeed() {
        setUpComponent();
    }

    private void setUpComponent() {
        String[] headings = {"Time sent/received","Sender","Receiver","Message"};

        model = new DefaultTableModel(headings,0) {
            @Override
            public boolean isCellEditable(int row, int column) {
                return false;
            }
        };

        setModel(model);

        setFocusable(false);
        setRowSelectionAllowed(false);

        getColumnModel().getColumn(0).setPreferredWidth(130);
        getColumnModel().getColumn(0).setMaxWidth(130);
        getColumnModel().getColumn(0).setMinWidth(130);
        getColumnModel().getColumn(0).setResizable(false);
        getColumnModel().getColumn(0).setHeaderValue("Time sent/received");

        getColumnModel().getColumn(1).setPreferredWidth(100);
        getColumnModel().getColumn(1).setMaxWidth(100);
        getColumnModel().getColumn(1).setMinWidth(100);
        getColumnModel().getColumn(1).setResizable(false);
        getColumnModel().getColumn(1).setHeaderValue("Sender");

        getColumnModel().getColumn(2).setPreferredWidth(100);
        getColumnModel().getColumn(2).setMaxWidth(100);
        getColumnModel().getColumn(2).setMinWidth(100);
        getColumnModel().getColumn(2).setResizable(false);
        getColumnModel().getColumn(2).setHeaderValue("Receiver");

        getColumnModel().getColumn(3).setMinWidth(450);
        getColumnModel().getColumn(3).setResizable(true);
        getColumnModel().getColumn(3).setHeaderValue("Message");

        setRowHeight(20);

        setVisible(true);
        setOpaque(true);
    }

    /**
     * Adds a row to the table containing specified data
     * @param message message content
     * @param sender sender agent address
     * @param recipient recipient agent address
     */
    public void addRow(String message, String sender, String recipient) {
        Object[] tableRow = new Object[4];

        tableRow[0] = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss").format(Calendar.getInstance().getTime());
        tableRow[1] = sender;
        tableRow[2] = recipient;
        tableRow[3] = message;

        model.addRow(tableRow);
    }
}
