
package middleware.components.useragents.messagesending;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import middleware.agents.UserAgent;
import middleware.components.useragents.messagefeed.MessageFeed;

/**
 * SendButton.
 * 
 * @author Dan Lawrence
 */
public class SendButton extends JButton implements ActionListener
{
    
    final RecipientBox recipientBox;
    
    final MessageBox messageBox;
    
    final MessageFeed messageFeed;
    
    private final UserAgent userAgent;
    
    /**
     * 
     * @param userAgent
     * @param recipientBoxIn
     * @param messageBoxIn
     * @param messageFeedIn 
     */
    public SendButton(UserAgent userAgent, RecipientBox recipientBoxIn, MessageBox messageBoxIn, MessageFeed messageFeedIn) {
        this.userAgent = userAgent;
        recipientBox = recipientBoxIn;
        messageBox = messageBoxIn;        
        messageFeed = messageFeedIn;
        
        setUpComponent();
    }
    
    private void setUpComponent() { 
        messageBox.sendButton = this;
        
        setPreferredSize(new Dimension(100, 20));
        setVisible(true);
        setOpaque(true);
        setText("Send");
        
        addActionListener(this);
    }
    
    @Override
    public void actionPerformed(ActionEvent e) {
        if ((e.getSource() == this) && !messageBox.getText().equals("") && !recipientBox.getText().equals("")) {
            sendMessage();
        }
    }
    
    void sendMessage() {
        String sender = userAgent.getAddress();
        String recipient = recipientBox.getText();
        String message = messageBox.getText();
                   
        userAgent.sendMsg(sender, recipient, message);
            
        messageFeed.addRow(message,sender,recipient);
        
        recipientBox.setText("");
        messageBox.setText("");
    }

}
