package middleware.message;

import java.io.Serializable;
import middleware.agents.AgentAddress;

/**
 * Message - sent between nodes
 * Containing information
 */
public abstract class Message implements Serializable
{
    private final Object message; //These can be objects
    private final AgentAddress recipient;
    
    /**
     * Creates an instance of Message
     * @param m Message Content
     * @param r Recipient
     */
    public Message(Object m, String r)
    {
        message = m;
        recipient = new AgentAddress(r);
    }
    
    /**
     * Gets message content
     * @return message content
     */
    public Object getMessage()
    {
        return message;
    }
    
    /**
     * Gets message recipient
     * @return recipient
     */
    public AgentAddress getRecipient()
    {
        return recipient;
    }
    
    /**
     * Gets sender
     * @return sender
     */
    public abstract AgentAddress getSender();
}
