package middleware.message;

import java.io.Serializable;

/**
 * Type of message used for communications between MetaAgents
 */
public class MetaAgentMessage implements Serializable
{
    private final Message message;
    
    /**
     * The maximum amount of hops a message can take before it is stopped
     */
    public int timeToLive;
    
    /**
     * The number of hops a message has taken
     */
    public int hops;
    
    /**
     * Creates an instance of MetaAgentMessage
     * Wraps an already created message
     * @param um message
     * @param ttl time to live
     */
    public MetaAgentMessage(Message um, int ttl)
    {
        hops = 0;
        message = um;
        timeToLive = ttl;
    }
    
    /**
     * Gets message
     * @return message
     */
    public Message getMessage()
    {
        return message;
    }
    
    /**
     * Gets message's TTL
     * @return TimeToLive
     */
    public int getTimeToLive()
    {
        return timeToLive;
    }
    
    /**
     * Gets no. of hops MetaAgentMessage has taken
     * @return hops
     */
    public int getHops()
    {
        return hops;
    }
    
    /**
     * Decrements TTL
     */
    public void decreaseTimeToLive()
    {
        timeToLive -= 1;
    }
    
    /**
     * Increments TTL
     */
    public void increaseHops()
    {
        hops += 1;
    }
}
