package middleware.message;

import middleware.agents.AgentAddress;

/**
 * User message
 * A type of message used between Portals (MetaAgent) and UserAgents
 */
public class UserMessage extends Message
{
    private final AgentAddress sender; //Can be object
    private final boolean needsResponse;
    private final String response; //Can be object
    private final String messageID;
    
    /**
     * Constructs an instance of UserMessage with specified properties
     * @param m message content
     * @param r recipient
     * @param s sender
     * @param n needs response?
     * @param res response content
     * @param id message ID
     */
    public UserMessage(Object m, String r, String s, boolean n, String res, String id)
    {
        super(m,r);
        sender = new AgentAddress(s);
        needsResponse = n;
        response = res;
        messageID = id;
    }
    
    /**
     * Gets a <i>UserMessage</i>'s sender
     * @return 
     */    
    @Override
    public AgentAddress getSender()
    {
        return sender;
    }
    
    /**
     * Gets whether <i>UserMessage</i> requires response
     * @return boolean, if needs response
     */
    public boolean getNeedsResponse()
    {
        return needsResponse;
    }
    
    /**
     * Gets the response for a <i>UserMessage</i>
     * @return Response
     */
    public String getResponse()
    {
        return response;
    }
    
    /**
     * Gets the message ID
     * @return message ID
     */
    public String getMessageID()
    {
        return messageID;
    }
}
