package middleware.nodemonitor;

import middleware.components.main.nodelistener.NodeListener;
import middleware.message.MetaAgentMessage;
import middleware.observer.Observer;

/**
 * NodeMonitor created to read messages received by a node (e.g. meta agent, agent) and add them to the GUI
 *
 * @author Dylan Currey
 * @since 12/12/18
 */
public class NodeMonitor implements Observer{

    
    private NodeListener nodeListener;
    
    /**
     * Creates an instance, using a passed GUI component
     * @param nodeListener a GUI component, to be updated by NodeMonitor
     */
    public NodeMonitor(NodeListener nodeListener)
    {
        this.nodeListener = nodeListener;
    }
    
    /**
     * Parses and updates GUI from received message
     * @param msg the message object received by the attached subject
     * @return whether operation was successful
     */
    @Override
    public boolean update(Object msg) {
        if(msg instanceof MetaAgentMessage)
        {
            MetaAgentMessage MAM = (MetaAgentMessage) msg;            
            nodeListener.addRow(MAM.getMessage().getMessage().toString(), MAM.getMessage().getSender().getAddress(), MAM.getMessage().getRecipient().getAddress(), MAM.getTimeToLive(), MAM.getHops());
            return true;
        }       
        return false;
    }
    
}
