package middleware.observer;

/**
 * A Subject interface created to facilitate the implementation of the Observer design pattern between UserAgents and NodeMonitors
 * 
 * @author Dylan Currey
 * @since 12/12/18
 */
public interface Subject
{
       
    /**
     * Attaches observers to the subject.
     * @param observer the <i>Observer</i> object to be attached.
     */
    public void attach(Observer observer);
    
    /**
     * Detaches observers from the subject
     * @param observer the <i>Observer</i> object to be detached
     */
    public void detach (Observer observer);
    
    /**
     * Updates all observers in <i>listOfObservers</i> with a command.
     * @param msg
     * @return 
     */
    public boolean updateObservers(Object msg);
    
}