package middleware.socket;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.net.Socket;
import middleware.message.MetaAgentMessage;

/**
 * Client
 * Acts as client in client-server relationship
 */
public class Client
{
    //Socket variables (for connection)
    private Socket socket;
    private String address;
    private int port;

    //Socket input and output variables
    private PrintWriter writer;
    private ObjectOutputStream objOut;

    /**
     * Creates an instance of Client
     * @param a address
     * @param p port
     */
    public Client(String a, int p)
    {
        address = a;
        port = p;
    }

    /**
     * Creates a connection with a socket and output stream
     * @throws IOException
     */
    public void connect() throws IOException
    {
        //Create new socket
        socket = new Socket(address, port);

        //Declare the output stream
        //writer = new PrintWriter(socket.getOutputStream());
        objOut = new ObjectOutputStream(socket.getOutputStream());
    }

    /**
     * Sends a String to a socket, may be used when communicating with other languages
     * @param s string to be sent
     */
    public void sendString(String s)
    {
        //TODO maybe?
    }

    /**
     * Sends serialisable objects (e.g. messages, in this case) to a socket
     * @param message message, serialisable to be sent
     * @throws IOException
     */
    public void send(MetaAgentMessage message) throws IOException
    {
        //Write the object/message to the stream
        objOut.writeObject((MetaAgentMessage) message);
        //Flush the stream, is nice to have
        objOut.flush();
    }

    /**
     * Closes all connections
     * @throws IOException
     */
    public void close() throws IOException
    {
        //Close all connections and writers
        socket.close();
        writer.close();
    }
}
