package middleware.socket;

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import middleware.agents.Router;

/**
 * Server acts as server in Client-Server Communications
 */
public class Server
{
    private ServerSocket serverSocket;
    private Socket socket;
    private int port;
    private InetAddress address;
    
    private Router router;

    private boolean listening;
    
    

    /**
     * Creates an instance of <i>Server</i> with given properties
     * @param p port
     */
    public Server(int p, Router r)
    {
        //address = a;
        port = p;
        router = r;

    }

    /**
     * Creates a new server socket on <i>Server</i>'s port
     * @throws IOException
     */
    public void create() throws IOException
    {
        //serverSocket = new ServerSocket(port, 50, address);
        serverSocket = new ServerSocket(port);
    }

    /**
     * Accepts input from socket
     * @throws IOException
     */
    private void listen() throws IOException
    {
        final ExecutorService clientProcessingPool = Executors.newFixedThreadPool(10);
        
        Runnable serverTask = new Runnable()
        {
            @Override
            public void run()
            {
                try
                {
                    for (;;)
                    {
                        socket = serverSocket.accept();
                        new ServerThread(socket, router).start();
                        System.out.println("Connected: " + socket.toString());
                    }
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
        };
        
        Thread serverThread = new Thread(serverTask);
        serverThread.start();
    }

    /**
     * <i>Server</i> starts listening to socket input
     * @throws IOException
     */
    public void enableListening() throws IOException
    {
        if (!listening)
        {
            listening = true;
            listen();
        }
    }

    /**
     * Stops <i>Server</i> listening to socket
     */
    public void disableListening()
    {
        listening = false;
    }
}
